<?php

namespace Drupal\campaignmonitor\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\Component\Utility\SafeMarkup;

/**
 * Configure campaignmonitor settings for this site.
 */
class CampaignMonitorListSettingsForm extends ConfigFormBase {


  /**
   * {@inheritdoc}
   */
  public function getFormID() {
    return 'campaignmonitor_list_settings_form';
  }

  protected function getEditableConfigNames() {
    return ['campaignmonitor.settings.list'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $list_id = NULL) {
    $config = $this->config('campaignmonitor.settings.list');

    $form = array('#tree' => TRUE);

    $defaults = campaignmonitor_get_list_settings($list_id);

    $list = campaignmonitor_get_extended_list_settings($list_id);

    // Add list id to the form.
    $form['listId'] = array(
      '#type' => 'hidden',
      '#value' => $list_id,
    );

    // Set this form name (index).
    $form_key = 'campaignmonitor_list_' . $list_id;

    $form[$form_key]['status'] = array(
      '#type' => 'fieldset',
      '#title' => t('Enable list'),
      '#description' => t('Enable the list to configure it and use it on the site.'),
      '#collapsible' => FALSE,
      '#collapsed' => FALSE,
    );

    $form[$form_key]['status']['enabled'] = array(
      '#type' => 'checkbox',
      '#title' => t('Enable'),
      '#default_value' => isset($defaults['status']['enabled']) ? $defaults['status']['enabled'] : 0,
      '#attributes' => array('class' => array('enabled-list-checkbox')),
    );

    $form[$form_key]['options'] = array(
      '#type' => 'fieldset',
      '#title' => t('List options'),
      '#description' => t('Changing the values will result in an update of the values on the Campaign Monitor homepage.'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      '#states' => array(
        'visible' => array(
          '.enabled-list-checkbox' => array('checked' => TRUE),
        ),
      ),
    );

    $form[$form_key]['options']['listname'] = array(
      '#type' => 'textfield',
      '#title' => t('List name'),
      '#default_value' => $list['name'],
      '#required' => TRUE,
      '#states' => array(
        'visible' => array(
          ':input[name="status[enabled]"]' => array('checked' => TRUE),
        ),
      ),
    );

    $form[$form_key]['options']['UnsubscribePage'] = array(
      '#type' => 'textfield',
      '#title' => t('Unsubscribe page'),
      '#default_value' => $list['details']['UnsubscribePage'],
    );

    $form[$form_key]['options']['ConfirmationSuccessPage'] = array(
      '#type' => 'textfield',
      '#title' => t('Confirmation success page'),
      '#default_value' => $list['details']['ConfirmationSuccessPage'],
    );

    $form[$form_key]['options']['ConfirmedOptIn'] = array(
      '#type' => 'checkbox',
      '#title' => t('Confirmed Opt In'),
      '#description' => t('Selecting this will mean that subscribers will need to confirm their email each time they
      subscribe to the list'),
      '#default_value' => $list['details']['ConfirmedOptIn'],
    );

    $form[$form_key]['display'] = array(
      '#type' => 'fieldset',
      '#title' => t('Display options'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      '#states' => array(
        'visible' => array(
          '.enabled-list-checkbox' => array('checked' => TRUE),
        ),
      ),
    );

    $form[$form_key]['display']['name'] = array(
      '#type' => 'checkbox',
      '#title' => t('Display Name field'),
      '#description' => t('Whether the Name field should be displayed when subscribing.'),
      '#default_value' => isset($defaults['display']['name']) ? $defaults['display']['name'] : 0,
      '#attributes' => array('class' => array('tokenable', 'tokenable-name')),
    );

    $form[$form_key]['display']['description'] = array(
      '#type' => 'textarea',
      '#title' => t('Description'),
      '#description' => t('A description to accompany the list in forms.'),
      '#default_value' => isset($defaults['display']['description']) ? $defaults['display']['description'] : '',
      '#attributes' => array('class' => array('tokenable', 'tokenable-description')),
    );

    $field_map = \Drupal::entityManager()->getFieldMap();

    $user_field_map = $field_map['user'];

    $user_fields = array_keys($user_field_map);
    foreach ($user_fields as $key => $user_field) {
      unset($user_fields[$key]);
      $user_fields[$user_field] = $user_field;
    }

    $form[$form_key]['display']['name_field'] = array(
      '#type' => 'select',
      '#options' => $user_fields,
      '#title' => t('Email name field'),
      '#description' => t('The name that will be used by Campaign Monitor as a salutation in emails sent out.'),
      '#default_value' => isset($defaults['display']['name_field']) ? $defaults['display']['name_field'] : 0,
    );




    // List custom fields.
    if (!empty($list['CustomFields'])) {
      $options = array();
      foreach ($list['CustomFields'] as $key => $field) {
        // Form API can't handle keys with [] in all cases.
        $token_form_key = str_replace(array('[', ']'), '', $key);
        $options[$token_form_key] = $field['FieldName'];
      }

      $form[$form_key]['CustomFields'] = array(
        '#type' => 'fieldset',
        '#title' => t('Custom fields'),
        '#collapsible' => TRUE,
        '#collapsed' => FALSE,
        '#attributes' => array('class' => array('tokenable', 'tokenable-custom-fields')),
        '#states' => array(
          'visible' => array(
            '.enabled-list-checkbox' => array('checked' => TRUE),
          ),
        ),
      );

      $form[$form_key]['CustomFields']['selected'] = array(
        '#type' => 'checkboxes',
        '#title' => t('Available fields'),
        '#description' => t('Select the fields that should be displayed on subscription forms.'),
        '#options' => $options,
        '#default_value' => isset($defaults['CustomFields']['selected']) ? $defaults['CustomFields']['selected'] : array(),
      );
    }

    if (\Drupal::moduleHandler()->moduleExists('token')) {
      $form[$form_key]['tokens'] = array(
        '#type' => 'fieldset',
        '#title' => t('Field tokens'),
        '#collapsible' => TRUE,
        '#collapsed' => FALSE,
      );

      $form[$form_key]['tokens']['name'] = array(
        '#type' => 'textfield',
        '#title' => t('Name field'),
        '#default_value' => isset($defaults['tokens']['name']) ? $defaults['tokens']['name'] : '[current-user:name]',
        '#states' => array(
          'visible' => array(
            '.tokenable-name' => array('checked' => TRUE),
          ),
        ),
      );

      if (!empty($list['CustomFields'])) {
        foreach ($list['CustomFields'] as $key => $field) {
          if ($field['DataType'] == 'MultiSelectMany') {
            // We can't handle this type of custom field (with tokens).
            continue;
          }

          // Form API can't handle keys with [] in all cases.
          $token_form_key = str_replace(array('[', ']'), '', $key);
          $form[$form_key]['tokens'][$token_form_key] = array(
            '#type' => 'textfield',
            '#title' => t('Custom field (@name)', array('@name' => $field['FieldName'])),
            '#default_value' => isset($defaults['tokens'][$token_form_key]) ? $defaults['tokens'][$token_form_key] : '',
            '#states' => array(
              'visible' => array(
                ':input[name="' . $form_key . '[CustomFields][selected][' . $token_form_key . ']' . '"]' => array('checked' => TRUE),
              ),
            ),
          );
        }
      }

      $form[$form_key]['tokens']['token_tree'] = array(
        '#theme' => 'token_tree',
      );
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   *
   * Edit form validation handler which calls the API to save the information that
   * was entered. This is done in the validation function so we can give better
   * feedback to the user and to prevent the user from having to enter the
   * information once more on failure.
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

    $list_id = $form_state->getValue('listId');
    $values = $form_state->getValues();
    // Build array with basic information.

    $values = $values['campaignmonitor_list_' . $list_id];
    $options = array(
      'Title' => SafeMarkup::checkPlain($values['options']['listname']),
      'UnsubscribePage' => SafeMarkup::checkPlain($values['options']['UnsubscribePage']),
      'ConfirmedOptIn' => $values['options']['ConfirmedOptIn'] ? TRUE : FALSE,
      'ConfirmationSuccessPage' => SafeMarkup::checkPlain($values['options']['ConfirmationSuccessPage']),
    );
    $result = campaignmonitor_set_extended_list_settings($list_id, $options);

    if ($result != 'success') {
      $form_state->setErrorByName('', $result);
    }

    // Redirect to list overview.
    $url = Url::fromRoute('campaignmonitor.lists');
    $form_state->setRedirectUrl($url);
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $list_id = $form_state->getValue('listId');
    $list_options = campaignmonitor_get_list_settings($list_id);

    $values = $form_state->getValues();
    $values = $values['campaignmonitor_list_' . $list_id];
    // These are saved remotely
    unset ($values['options']);

    campaignmonitor_set_list_settings($list_id, $values);

    parent::submitForm($form, $form_state);
  }

}

